#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/04/22 18:12:55
'''


class UnittestHelperError(Exception):
    pass


class AttributeMarkerException(UnittestHelperError):
    pass


class ConstAttributeException(UnittestHelperError):
    pass



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
'''

import os
import inspect
import functools
import collections.abc as collections

from .global_config import GLOBAL_CONFIG
from .abstract_data_provider import AbsractDataProvider
from .seven_data_provider import SevenDataProvider


class Test(object):
    """将方法标记为测试方法

    Args:
        settings: 设置项, 键定义如下
            author: 用例编写者
            editors: 修改者列表
            dname: 用于给用例起一个用于设置依赖的名称
            depends: 用于设置用例依赖，是一个用例依赖列表
            groups: 方法所属的组的列表
            enabled: 是否启用执行该测试方法
            priority: 测试方法的执行优先级，数值越小执行越靠前
            alway_run: 如果设置为True，不管依赖它所依赖的其他用例结果如何都始终运行，为False时，则它所依赖的其他用例不成功，就不会执行，默认值为False
            description: 测试用例名称
            data_provider: 测试方法的参数化数据提供者，AbsractDataProvider的子类或者一个可调用的对象，返回数据集列表
            测试方法只有一个参数化时，data_provider 返回值是 一维列表，如:
                def ia_data_provider(testclass, testmethod, *args, **kwargs):
                    return [1,2,3,4,5]
                @testcase(priority=1, enabled=True, data_provider=ia_data_provider, author='思文伟', description='整数加法测试01')
                def integer_addition_01(self, testdata):
                    print(testdata)

            测试方法有多个参数化时，data_provider 返回值是 二维列表，如:
                def ia_data_provider(testclass, testmethod, *args, **kwargs):
                    return [[1,2],[3,4],[5,6]]
                @testcase(priority=1, enabled=True, data_provider=ia_data_provider, author='思文伟', description='整数加法测试01')
                def integer_addition_01(self, testdata01, testdata_02):
                    print(testdata01)
                    print(testdata_02)

            data_provider_args: 数据提供者变长位置参数(args)
            data_provider_kwargs: 数据提供者变长关键字参数(kwargs)
            last_modifyied_by: 最后修改者
            last_modified_time: 最后一次修改的时间
            enable_default_data_provider: 是否使用内置数据提供者(SevenDataProvider)，未设置data_provider，且该值为True 才会使用内置数据提供者(SevenDataProvider)
    """

    # 添加到测试方法的属性名
    TEST_MARKER = 'test_settings'  # 属性值是一个字典
    COLLECT_TEST_DATASETS = 'collect_test_datasets'  # 一个函数，搜集该方法的参数化测试数据
    ORIGINAL_TEST_METHOD_ARGSPEC = 'original_test_method_argspec'  # inspect.getfullargspec(original_test_method)

    # TEST_MARKER键名
    AUTHOR = 'author'
    DNAME = 'dname'  # 该键名用于给用例起一个用于设置依赖的名称
    DEPENDS = 'depends'  # 该键名用于设置用例依赖，是一个用例依赖列表
    EDITORS = 'editors'
    GROUPS = 'groups'
    ENABLED = 'enabled'
    PRIORITY = 'priority'
    ALWAY_RUN = 'alway_run'
    TEST_CLASS = 'test_class'
    TEST_METHOD = 'test_method'
    TEST_DATASETS = 'test_datasets'
    DESCRIPTION = 'description'
    DATA_PROVIDER = 'data_provider'
    DATA_PROVIDER_ARGS = 'data_provider_args'
    DATA_PROVIDER_KWARGS = 'data_provider_kwargs'
    LAST_MODIFYIED_BY = 'last_modifyied_by'
    LAST_MODIFYIED_TIME = 'last_modified_time'
    ENABLE_DEFAULT_DATA_PROVIDER = 'enable_default_data_provider'

    _FINAL_ENABLE_DEFAULT_DATA_PROVIDER = '_final_enable_default_data_provider'

    DEFAULT_DATA_PROVIDER = SevenDataProvider
    DEFAULT_DATA_PROVIDER_INIT_KWARGS = {
        SevenDataProvider.PARAM_DATA_FILE_NAME: None,
        SevenDataProvider.PARAM_DATA_FILE_DIR_PATH: None,
        SevenDataProvider.PARAM_SHEET_NAME_OR_INDEX: 0,
    }

    # 默认配置
    DEFAULT_SETTTINGS = {
        GROUPS: [],
        DNAME: "",
        DEPENDS: [],
        ENABLED: True,
        PRIORITY: 1,
        ALWAY_RUN: False,
        TEST_CLASS: None,  # 该值由该类自动设置，不需要用户设置(The value are automatically seted by this class, and do not need to seted by user)
        TEST_METHOD: None,  # 该值由该类自动设置，不需要用户设置(The value are automatically seted by this class, and do not need to seted by user)
        DESCRIPTION: '',
        TEST_DATASETS: [],  # 该值由该类自动设置，不需要用户设置(The value are automatically seted by this class, and do not need to seted by user)
        DATA_PROVIDER: None,
        DATA_PROVIDER_ARGS: (),
        DATA_PROVIDER_KWARGS: {},
        AUTHOR: '',
        EDITORS: [],
        LAST_MODIFYIED_BY: '',
        LAST_MODIFYIED_TIME: '',
        ENABLE_DEFAULT_DATA_PROVIDER: True,
        _FINAL_ENABLE_DEFAULT_DATA_PROVIDER: False,  # 内置标志位，用于判断最终使用的是默认的data provider 还是调用者提供的，该值由该类自动设置，不需要用户设置
    }

    def __init__(self, **settings):
        self.settings = {k: v for k, v in self.DEFAULT_SETTTINGS.items()}
        settings.pop(self.TEST_CLASS, None)
        settings.pop(self.TEST_METHOD, None)
        settings.pop(self.TEST_DATASETS, None)
        settings.pop(self._FINAL_ENABLE_DEFAULT_DATA_PROVIDER, None)
        self.settings.update(settings)

        self.enable_parametrize = False

        # 未设置data_provider同时设置enable_default_data_provider = True,才会启用默认的数据提供者
        if self.settings.get(self.ENABLE_DEFAULT_DATA_PROVIDER) and (not self.settings.get(self.DATA_PROVIDER, None)):
            self.settings[self.DATA_PROVIDER] = self.DEFAULT_DATA_PROVIDER
            self.settings[self._FINAL_ENABLE_DEFAULT_DATA_PROVIDER] = True

    def get_testclass_file_dirpath(self, testclass):

        module = inspect.getmodule(testclass)
        filepath = os.path.abspath(module.__file__)
        dirpath = os.path.dirname(filepath)
        return dirpath

    def get_testmethod_file_dirpath(self, testmethod):
        """获取测试方法所在模块的目录路径

        Args:
            testmethod: 测试方法对象
        """

        module = inspect.getmodule(testmethod)
        filepath = os.path.abspath(module.__file__)
        dirpath = os.path.dirname(filepath)
        return dirpath

    def collect_test_datasets(self, testclass, testmethod):
        """调用参数化数据提供者，获取测试方法的参数化测试数据集

        Args:
            testclass: 被装饰的测试方法所在的测试类
            testmethod: 被装饰的测试方法
        Returns: 返回测试数据集，一个列表，每一个元素是该测试方法每次运行的测试数据
        """

        self.settings[self.TEST_CLASS] = testclass
        self.settings[self.TEST_METHOD] = testmethod
        data_provider = self.settings[self.DATA_PROVIDER]

        if not self.enable_parametrize:
            return
        if data_provider is None:
            return self.settings[self.TEST_DATASETS]

        if inspect.isclass(data_provider) and issubclass(data_provider, AbsractDataProvider):
            if issubclass(data_provider, SevenDataProvider) or data_provider == SevenDataProvider:
                kwargs = {k: v for k, v in self.DEFAULT_DATA_PROVIDER_INIT_KWARGS.items()}
                dp_kwargs = self.settings[self.DATA_PROVIDER_KWARGS]
                for k in kwargs.keys():
                    if k in dp_kwargs:
                        kwargs[k] = dp_kwargs[k]
                dp_kwargs.update(kwargs)

                # 如果没有设置测试数据文件所在的目录路径，则到全局配置去找
                # 全局配置里，也没有设置，则自动获取测试方法所在的模块的目录作为测试数据文件的查找目录
                if not dp_kwargs.get(SevenDataProvider.PARAM_DATA_FILE_DIR_PATH, None):
                    if GLOBAL_CONFIG.seven_data_provider_data_file_dir:
                        dp_kwargs[SevenDataProvider.PARAM_DATA_FILE_DIR_PATH] = GLOBAL_CONFIG.seven_data_provider_data_file_dir
                    else:
                        dp_kwargs[SevenDataProvider.PARAM_DATA_FILE_DIR_PATH] = self.get_testmethod_file_dirpath(testmethod)

                # 未提供测试数据文件名称则以测试方法所属的类的类名作为测试数据文件名称
                if not dp_kwargs.get(SevenDataProvider.PARAM_DATA_FILE_NAME, None):
                    dp_kwargs[SevenDataProvider.PARAM_DATA_FILE_NAME] = testclass.__name__

                self.settings[self.DATA_PROVIDER_KWARGS] = dp_kwargs
                datasets = data_provider().get_testdatas(testclass.__name__, testmethod.__name__, *self.settings[self.DATA_PROVIDER_ARGS], **self.settings[self.DATA_PROVIDER_KWARGS])
            else:
                datasets = data_provider().get_testdatas(testclass.__name__, testmethod.__name__, *self.settings[self.DATA_PROVIDER_ARGS], **self.settings[self.DATA_PROVIDER_KWARGS])

        elif self._validate_data_provider(data_provider):
            datasets = data_provider(testclass.__name__, testmethod.__name__, *self.settings[self.DATA_PROVIDER_ARGS], **self.settings[self.DATA_PROVIDER_KWARGS])
        else:
            raise TypeError('无效data_provider: {}'.format(data_provider))

        if not self._validate_datesets(datasets):
            # 返回的数据集必须是列表或者元祖类型, 实际返回的类型是
            raise TypeError('{}The returned data set must be a list or tuple type. The actual returned type is: {}'.format(data_provider, type(datasets)))

        self.settings[self.TEST_DATASETS] = datasets
        return datasets

    def _validate_data_provider(self, data_provider):

        return inspect.ismethod(data_provider) or inspect.isfunction(data_provider) or isinstance(data_provider, collections.Callable)

    def _validate_datesets(self, datasets):

        return isinstance(datasets, (list, tuple))

    # def test_datasets_filedir_path(self, module='__main__'):

    # if isinstance(module, str):
    # self.module = __import__(module)
    # for part in module.split('.')[1:]:
    # self.module = getattr(self.module, part)
    # else:
    # self.module = module
    # file_dir_path = os.path.dirname(os.path.abspath(self.module.__file__))

    def __call__(self, func):

        func_name = func.__name__
        argspec = inspect.getfullargspec(func)
        setattr(func, self.TEST_MARKER, self.settings)
        setattr(func, self.COLLECT_TEST_DATASETS, self.collect_test_datasets)
        setattr(func, self.ORIGINAL_TEST_METHOD_ARGSPEC, argspec)

        # 测试方法只有一个位置参数self,则不会执行参数化
        if len(argspec.args) <= 1:
            self.enable_parametrize = False
        else:
            self.enable_parametrize = True

        @functools.wraps(func)
        def _call(*args, **kwargs):

            instance = args[0] if len(args) > 0 else None
            method_instance = instance if instance and self.is_method_instance(instance, func_name) else None
            new_args = list(args)

            # argspec.args 位置参数
            pos_args_len = len(argspec.args)

            # argspec.varargs 变长位置参数,定义有则返回定义的名称否则None
            # 如 def connet(ip, *other) --> 返回other
            pos_varargs_len = 1 if argspec.varargs else 0

            insert_data = False
            insert_pos_index = 0

            if pos_varargs_len <= 0:
                if pos_args_len <= 0:
                    insert_data = False
                elif pos_args_len < 2:
                    if method_instance:
                        insert_data = False
                    else:
                        insert_data = True
                        insert_pos_index = 0
                else:
                    insert_data = True
                    if method_instance:
                        insert_pos_index = 1
                    else:
                        insert_pos_index = 0
            else:
                insert_data = True
                if method_instance:
                    insert_pos_index = 1
                else:
                    insert_pos_index = 0

            if insert_data:
                test_datasets = self.settings[self.TEST_DATASETS]
                runtime_testdatas, found, msg = self._get_runtime_testdatas(test_datasets, method_instance, func_name)

                if self.settings[self.DATA_PROVIDER]:
                    if found:
                        # 计算测试方法需要传入的参数个数
                        need_params_total = pos_args_len
                        # 如果是方法实例, 参数总数-1 如 def test_plus(self, testdata) --> 需要传入的参数是 testdata  就是1个
                        if method_instance:
                            need_params_total = need_params_total - 1

                        if need_params_total == 1:
                            new_args.insert(insert_pos_index, runtime_testdatas)
                        elif need_params_total > 1:
                            if isinstance(runtime_testdatas, (list, tuple)):
                                actual_params_total = len(runtime_testdatas)
                            else:
                                actual_params_total = 1

                            # 实际等到的参数个数大于等于需要的参数个数,
                            # 则传入需要位置索引一样的参数值, 多余的丢弃
                            # 实际等到的参数个数小于需要的参数个数，则报错
                            if actual_params_total >= need_params_total:
                                for i in range(need_params_total):
                                    new_args.insert(insert_pos_index + i, runtime_testdatas[i])
                            else:
                                raise TypeError('{}() need {} positional argument, but only {} were given'.format(func_name, need_params_total, actual_params_total))
                        else:
                            pass
                    else:
                        raise ValueError(msg)
            if method_instance:
                method_instance.set_testcase_runtime_datas(new_args, kwargs)
            return func(*tuple(new_args), **kwargs)

        return _call

    def _get_runtime_testdatas(self, test_datasets, test_method_instance, test_method_name):
        """从方法的测试数据集中获取本次测试方法运行时的测试数据

        Args:
            test_datasets: 测试方法的测试数据集，一个列表，每一个元素是该测试方法每次运行的测试数据
            test_method_instance: 测试方法所属的测试用例对象（即方法所在的测试类的实例化）
            test_method_name: 测试方法名称

        Returns: 返回一个3个元素的元祖，第一个是本次测试方法运行时的测试数据，第二个是查找测试数据的结果(False - 未找到, True - 找到)，第三是相关信息
        """

        runtime_datas = None
        found = False
        msg = ""
        if test_method_instance:
            sn = test_method_instance._serial_number
            for i, v in enumerate(test_datasets):
                if (i + 1) == sn:
                    runtime_datas = v
                    found = True
                    break
            if not found:
                msg = 'The parameterized test data for this method({}) was not found'.format(test_method_instance.id())
        else:
            pass
        return (runtime_datas, found, msg)

    def is_method_instance(self, instance, method_name):
        return inspect.ismethod(getattr(instance, method_name))

    @classmethod
    def func_has_test_marker(cls, obj):
        """判断对象是否有测试标志"""

        if inspect.ismethod(obj) or inspect.isfunction(obj):
            has_marker = getattr(obj, cls.TEST_MARKER, None)
            if has_marker:
                return True
            closure = getattr(obj, '__closure__', None)
            c_func = None
            if closure:
                for c in closure:
                    contents = c.cell_contents
                    if inspect.ismethod(contents) or inspect.isfunction(contents):
                        c_func = contents
                        break
            if c_func and getattr(c_func, cls.TEST_MARKER, None):
                return True
        else:
            return False

    @classmethod
    def get_test_marker(cls, test_func_obj, key=None, default_value=None):
        """返回测试标记配置或者配置中的某个键值

        Args:
            test_func_obj: 测试方法对象
            key: 要获取测试标记配置项的键名，如果不设置则返回整个测试标记配置
            default_value: 键值项不存在时的默认返回值
        Returns: 返回整个测试标记配置 或配置中指定项的值
        """

        test_marker = getattr(test_func_obj, cls.TEST_MARKER, {})
        if key and isinstance(key, str):
            return test_marker.get(key, default_value)
        else:
            return test_marker



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
'''

import os
import re
import json
import inspect
import tkinter
import smtplib
import functools
import tkinter.simpledialog
import collections.abc as collections
from functools import cmp_to_key
from email.header import Header
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart


def strclass(cls):

    module = inspect.getmodule(cls)
    mfile = inspect.getabsfile(module)
    name = inspect.getmodulename(mfile)
    return "%s.%s" % (name, cls.__qualname__)


def ignore_unused(*objs):
    return objs


def is_number(self, text):

    number = re.compile(r'^[+]?[0-9]+\.[0-9]+$')
    result = number.match(text)
    if result:
        is_number = True
    else:
        is_number = False

    return is_number


def is_string(value):

    return isinstance(value, (str))


def is_positive_integer(value):

    return isinstance(value, int) and value >= 0


def is_empty_string(value):

    return is_string(value) and value == ""


def is_blank_space(value):

    return is_string(value) and value.strip() == ""


def group_by_suffix_regex(string_key_dict, suffix_regex, is_remove_suffix=True, cmp=None):
    """根据键名正则表达式对字典进行分组和排序

    Args:
        string_key_dict 键名是字符串的字典
        suffix_regex 键名正则表达式
        is_remove_suffix 是否移除匹配的后缀
        cmp 如果是函数，会往cmp传入匹配的后缀，函数要求同sorted方法的cmp参数说明，如果是为True则按照匹配的后缀排序，False则不排序
    Returns: 返回分组后组构成的列表，每个组是一个字典 [group, group, ...]
    """

    regex = "(.)*" + "(" + suffix_regex + ")"
    pattern = re.compile(regex)

    unique_suffixes = []
    suffix_key_val_maps = []
    for k, v in string_key_dict.items():
        matcher = pattern.search(k)
        if matcher:
            full_key_name = matcher.group(0)
            key_name_suffix = matcher.group(2)
            suffix_key_val_maps.append((key_name_suffix, full_key_name, v))
            if key_name_suffix not in unique_suffixes:
                unique_suffixes.append(key_name_suffix)

    # sorted
    if inspect.isfunction(cmp):
        unique_suffixes = sorted(unique_suffixes, key=cmp_to_key(cmp))
    elif cmp:
        unique_suffixes = sorted(unique_suffixes)

    # group
    groups = []
    for us in unique_suffixes:
        group = {}
        for suffix, key, val in suffix_key_val_maps:
            if us == suffix:
                if is_remove_suffix:
                    k = key[:len(key) - len(suffix)]
                else:
                    k = key
                group[k] = val
        if group:
            groups.append(group)
    return groups


def digital_extractor(extractor=None):
    """数字提取函数装饰器,把被装饰的函数变成符合sorted要求的cmp函数

    Args:
        extractor 数字提取函数 需要接受一个字符串参数，从中提取数字并返回
    """
    def wapper(func):
        @functools.wraps(func)
        def recv_args(suffix1, suffix2):
            return func(suffix1, suffix2, extractor)

        return recv_args

    return wapper


@digital_extractor()
def digital_suffix_cmp(suffix1, suffix2, digital_extractor=None):
    """数字后缀比较器

    Args:
        suffix1 以数字结尾的字符串1
        suffix2 以数字结尾的字符串2
        digital_extractor 数字提取函数 需要接受一个字符串参数，从中提取数字并返回
    """
    def _digital_extractor(suffix):
        regex = "(\\d+)$"
        pattern = re.compile(regex)
        matcher = pattern.search(suffix)
        if not matcher:
            message = "所给的后缀" + "(" + suffix + ")" + "没有以数字结尾"
            raise ValueError(message)
        return matcher.group(1)

    if digital_extractor and inspect.isfunction(digital_extractor):
        extractor = digital_extractor
    else:
        extractor = _digital_extractor

    n1 = int(extractor(suffix1))
    n2 = int(extractor(suffix2))

    if n1 > n2:
        return 1
    elif n1 == n2:
        return 0
    else:
        return -1


def digital_suffix_cmp_wrapper(digital_extractor=None):
    """ 数字后缀比较器包装器

    Args:
        digital_extractor 数字提取函数 需要接受一个字符串参数，从中提取数字并返回
    """

    if digital_extractor and inspect.isfunction(digital_extractor):

        extractor = digital_extractor

        def new_digital_suffix_cmp(suffix1, suffix2):

            n1 = int(extractor(suffix1))
            n2 = int(extractor(suffix2))

            if n1 > n2:
                return 1
            elif n1 == n2:
                return 0
            else:
                return -1

        return new_digital_suffix_cmp
    else:
        return digital_suffix_cmp


def prompt(title="", tips="", encoding="utf-8"):
    """用于显示可提示用户进行输入的对话框

    Args:
        title 对话框标题
        tips 要在对话框中显示的文本
    """

    root = tkinter.Tk()
    screen_width = root.winfo_screenwidth()
    screen_height = root.winfo_screenheight() - 100  # under windows, taskbar may lie under the screen
    root.resizable(False, False)
    root.withdraw()
    # 移到屏幕外，避免闪烁
    root.geometry('+%d+%d' % (screen_width + 100, screen_height + 100))
    root.update_idletasks()
    # root.deiconify()    #now window size was calculated
    # root.withdraw()     #hide window again
    root.geometry('%sx%s+%s+%s' % (root.winfo_width() + 10, root.winfo_height() + 10, (screen_width - root.winfo_width()) / 2, (screen_height - root.winfo_height()) / 2))  # center window on desktop
    root.update()
    root.withdraw()

    # root.deiconify()
    # add some widgets to the root window
    if not isinstance(title, bytes):
        title = title.decode(encoding)
    if not isinstance(tips, bytes):
        tips = tips.decode(encoding)
    value = tkinter.simpledialog.askstring(title, tips)
    if value is None:
        value = ""
    return value


def escape_xpath_value(value):
    if '"' in value and '\'' in value:
        parts_wo_apos = value.split('\'')
        return "concat('%s')" % "', \"'\", '".join(parts_wo_apos)
    if '\'' in value:
        return "\"%s\"" % value
    return "'%s'" % value


class DictToObject(object):
    def __init__(self, py_dict):
        """

        """
        if not isinstance(py_dict, dict):
            raise ValueError("Data Type Must Be dict")
        for field in py_dict:
            value = py_dict[field]
            if not isinstance(value, (dict, list, tuple)):
                setattr(self, "%s" % field, value)
            else:
                if isinstance(value, dict):
                    setattr(self, "%s" % field, self.__class__(value))
                elif isinstance(value, tuple):
                    setattr(self, "%s" % field, self.analyze_tuple(value))
                else:
                    setattr(self, "%s" % field, self.analyze_list(value))

    def analyze_list(self, array):
        for index, value in enumerate(array):
            if isinstance(value, list):
                self.analyze_list(value)
            elif isinstance(value, tuple):
                self.analyze_tuple(value)
            elif isinstance(value, dict):  # if element of list is dict,converts the value of the element in the list into the object
                array[index] = self.__class__(value)
            else:
                pass
        return array

    def analyze_tuple(self, array):
        tuple_to_list = list(array)
        for index, value in enumerate(tuple_to_list):
            if isinstance(value, tuple):
                self.analyze_tuple(value)
            elif isinstance(value, tuple):
                self.analyze_tuple(value)
            elif isinstance(value, dict):  # if element of list is dict,converts the value of the element in the list into the object
                tuple_to_list[index] = self.__class__(value)
            else:
                pass
        return tuple(tuple_to_list)


def get_the_number_of_pages(total_data, limit_size):
    """根据数据总数和每页显示数据数计算分页数

    Args:
        total_data 数据总数
        limit_size 每页显示数据条数
    """
    quotient, remainder = divmod(total_data, limit_size)
    if remainder > 0:
        quotient = quotient + 1
    return quotient


def cutout_prefix_digital(string):

    regex = "^(\\d+)\\.*"
    pattern = re.compile(regex)
    matcher = pattern.search(string)
    if matcher:
        return matcher.group(1)
    else:
        return None


def find_same_and_diff_fields(fields_1, fields_2, distinct=True):
    """找出fields_1和fields_2中都存在的字段；fields_1中有，但fields_2中没有的字段；
    fields_2中有，但fields_1中没有的字段

    Args:
        fields_1: 字段列表1
        fields_1: 字段列表2
        distinct: 是否去除返回的3个列表中重复的字段, True则去重

    Returns:
        same: list 都存在的相同字段
        redundant: list fields_1中有，但fields_2中没有的字段
        missing: fields_2中有，但fields_1中没有的字段
    """

    same = []
    redundant = []
    missing = []
    for field in fields_1:
        if field in fields_2:
            if distinct:
                if field not in same:
                    same.append(field)
            else:
                same.append(field)
        else:
            if distinct:
                if field not in redundant:
                    redundant.append(field)
            else:
                redundant.append(field)

    for field in fields_2:
        if field not in fields_1:
            if distinct:
                if field not in missing:
                    missing.append(field)
            else:
                missing.append(field)

    return same, redundant, missing


def isfilelike(f):
    """
    Check if object 'f' is readable file-like
    that it has callable attributes 'read' , 'write' and 'close'
    """
    try:
        if isinstance(getattr(f, "read"), collections.Callable) and isinstance(getattr(f, "write"), collections.Callable) and isinstance(getattr(f, "close"), collections.Callable):
            return True
    except AttributeError:
        pass
    return False


def send_email(sender, password, subject="自动化测试报告", message="", mail_to=[], mail_cc=[], mail_bcc=[], attachment=None, smtpserver="smtp.qiye.163.com", port=0, debug=False):
    """

    Args:
        sender:发件人邮箱地址
        password:密码
        mail_to:主送人邮箱列表
        mail_cc:抄送人邮箱列表
        mail_bcc:密送人邮箱列表
        message:邮件正文
        attachment: 附件完整路径字符串或一个以二进制模式打开的文件对象
    """
    resend_time = 1
    msg = MIMEMultipart("mixed")
    altmsg = MIMEMultipart("alternative")
    message = '<pre style="font-weight: 500;font-size: 16px;color: darkcyan;">%s</pre>' % message
    body = MIMEText(message, _subtype='html', _charset='utf8')
    altmsg.attach(body)
    if attachment:
        if isinstance(attachment, str):
            f = open(attachment, 'rb')
            file_name = os.path.split(attachment)[-1]
        elif isfilelike(attachment):
            f = attachment
            file_name = os.path.split(f.name)[-1]
        else:
            raise TypeError("attachment must be file object or string (full path of file)")
        f_content = f.read()
        f.close()

        if file_name.endswith(".html"):
            html = MIMEText(f_content, _subtype='html', _charset='utf-8')
            altmsg.attach(html)
        att = MIMEText(f_content, "base64", _charset='utf8')
        att["Content-Type"] = 'application/octet-stream;charset=utf-8'
        att.add_header("Content-Disposition", "attachment", filename=Header(file_name, 'utf-8').encode())
        # att["Content-Disposition"]  = 'attachment;filename="%s"' % file_name
        # att = MIMEBase('application', 'octet-stream')
        # att.set_payload(f_content)
        # att.add_header('Content-Disposition', 'attachment', filename=('gbk', '', file_name) )
        # encoders.encode_base64(att)
        msg.attach(att)
    msg.attach(altmsg)
    if mail_to:
        msg["To"] = ",".join(mail_to)
    if mail_cc:
        msg["Cc"] = ",".join(mail_cc)
    if mail_bcc:
        msg["Bcc"] = ",".join(mail_bcc)
    receiver = mail_to + mail_cc + mail_bcc
    msg["from"] = sender
    msg["Subject"] = Header(subject, "utf8")
    msg["Accept-Language"] = "zh-CN"
    msg["Accept-Charset"] = "ISO-8859-1,utf-8"

    smtp_sender = smtplib.SMTP(host=smtpserver, port=port)
    # smtp_sender.connect(smtpserver)
    smtp_sender.ehlo()
    smtp_sender.starttls()
    is_success = True
    while True:
        try:
            smtp_sender.login(sender, password)
            smtp_sender.sendmail(sender, receiver, msg.as_string())
            is_success = True
            break
        except smtplib.SMTPException as e:
            resend_time = resend_time + 1
            is_success = False
            if resend_time == 3:
                if debug:
                    print("send email %s times failed" % resend_time)
                    print(e)
                break

    smtp_sender.quit()
    return is_success


def to_json_obj(json_str):

    return json.loads(json_str)


def to_json_str(json_obj):

    return json.dumps(json_obj, ensure_ascii=False)


def get_caller_name():
    """获取直接调用该函数的函数或方法名"""

    return inspect.stack()[1][3]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/15
'''
from unittest.runner import TextTestResult


class SevenTestResult(TextTestResult):
    def __init__(self, stream, descriptions, verbosity, depend_manager=None):
        super().__init__(stream, descriptions, verbosity)
        self.successes = []
        self.depend_manager = depend_manager

    def startTest(self, test):

        return super().startTest(test)

    def stopTest(self, test):
        return super().stopTest(test)

    def addSuccess(self, test):

        return_value = super().addSuccess(test)
        self.successes.append(test)
        return return_value

    def getDescription(self, test):

        short_desc = test.shortDescription()
        if self.descriptions and short_desc:
            return '\n'.join((str(test), short_desc))
        else:
            return str(test)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/18
'''
from .result_formatter import TestResultFormatter
from .report.htmltemplate import HtmlReportTemplate


class ReportBuilder(object):
    def __init__(self, result):

        self.result = result

    def build_html_report(self, filename, **summary_info):
        """

        Args:
            filename: 报告文件名（完整文件路径名） eg: E:\\TEST\\autotest.html

            summary_info:
                title: 报告标题
                task_number: 测试任务编号
                start_time: 测试开始时间
                finish_time: 测试完成时间
                executor: 测试任务执行者
                project_name: 项目名称
                task_description: 测试任务描述信息
        """

        json_result = TestResultFormatter(self.result).to_py_json()
        testpoints = json_result.get("testpoints")
        template = HtmlReportTemplate(testpoints, **summary_info)
        template.save_as_file(filename)



# -*- coding: utf-8 -*-
'''
Created on 2019年12月5日

@author: siwenwei
'''


class Tags(object):

    html = "html"

    meta = "meta"

    title = "title"

    style = "style"

    script = "script"

    head = "head"

    body = "body"

    table = "table"

    thead = "thead"

    tbody = "tbody"

    th = "th"

    tr = "tr"

    td = "td"

    div = "div"

    span = "span"

    label = "label"

    fieldset = "fieldset"

    legend = "legend"

    pre = "pre"

    svg = "svg"

    path = "path"

    text = "text"

    rect = "rect"

    circle = "circle"

    dl = "dl"

    dt = "dt"

    dd = "dd"

    input = "input"

    ul = "ul"

    li = "li"

    textarea = "textarea"

    italic = "i"

    p = "p"

    hr = "hr"



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/23
'''
import inspect
import collections
from .attrs_manager import AttributeManager
from .attrs_marker import ConstAttributeMarker


class TestCaseWrapper(AttributeManager):

    SEP = ConstAttributeMarker(".", "id分隔符")
    SN = ConstAttributeMarker("serial_number", "AbstractTestCase的子类会有个用例序列号属性serial_number，用来实现参数化的")
    SN_PREFIX = ConstAttributeMarker("_", "serial_number分隔符")
    PY = ConstAttributeMarker(".py", "Python 模块文件拓展名")
    SETTINGS_KEY = ConstAttributeMarker("test_method_settings", "测试方法存储设置项的属性名")
    DEPENDS_KEY = ConstAttributeMarker("depends", "测试方法设置项中设置依赖的键名")
    DNAME_KEY = ConstAttributeMarker("dname", "测试方法设置命名用于依赖的键名")

    def __init__(self, testcase):

        self._testcase = testcase
        self._testsettings = getattr(self._testcase, self.SETTINGS_KEY, {})
        self._serial_number = getattr(self._testcase, self.SN, None)
        self._init()
        self._parse()

    def _init(self):

        self._new_id = None

        self._module = None
        self._module_file = None
        self._module_real_name = None

        self._module_scope = None
        self._class_scope = None
        self._module_file_name = None

    @property
    def testcase(self):

        return self._testcase

    def id(self):
        """测试用例原id"""

        return self.testcase.id()

    @property
    def serial_number(self):

        return self._serial_number

    @property
    def settings(self):

        return self._testsettings

    @property
    def dnames(self):
        """用来设置依赖的命名"""

        name = self._testsettings.get(self.DNAME_KEY, '')
        if name and isinstance(name, str):
            return [name]
        return name

    @property
    def depends(self):

        return self.settings.get(self.DEPENDS_KEY, [])

    @property
    def module_scope(self):
        """生成的新的测试用例id中模块部分完整域名 test_module.py"""

        return self._module_scope

    @property
    def class_scope(self):
        """生成的新的测试用例id中类部分的完整域名:  test_module.py.TestClass"""

        return self._class_scope

    @property
    def new_id(self):
        """将测试用例原始ID转换为具有模块扩展名的新ID（convert test case original ID  to new ID with module extend name）,如果是用例是AbstractTestCase子类的实例，还会移除id末尾序列号

        原来的ID大体由3部分构成: 模块名.类.测试方法名，新的为 模块文件名.类.测试方法名

        test_module.TestClass.test --> test_module.py.TestClass.test
        """

        return self._new_id

    @property
    def module_real_name(self):
        """测试用例所在的模块名称（不含文件拓展名）"""

        return self._module_real_name

    @property
    def module_file_name(self):
        """测试用例所在的模块文件名，如果找不到模块的实际名称则返回测试用例id中模块名部分,
        id中的模块名部分是调用类的__module__来获取的，则有可能为__main__
        """

        return self._module_file_name

    def _parse(self):

        self._module = inspect.getmodule(self.testcase)
        self._module_file = inspect.getabsfile(self._module)
        self._module_real_name = inspect.getmodulename(self._module_file)

        parts = self.id().split(self.SEP)
        if len(parts) < 3:
            raise ValueError('Invalid Test Case ID: {}'.format(self.id()))
        original_module = parts[0]
        original_clazz = self.SEP.join(parts[1:-1])
        original_testname = parts[-1]

        module_name = self._module_real_name if self._module_real_name else original_module
        self._module_file_name = module_name + self.PY

        self._module_scope = self._module_file_name
        self._class_scope = self.SEP.join([self._module_file_name, original_clazz])
        self._new_id = self.SEP.join([self._module_file_name, original_clazz, original_testname])

        # 如果是用例是AbstractTestCase子类的实例，则移除序列号
        if self.serial_number is not None:
            self._new_id = self._new_id.rsplit(self.SN_PREFIX, 1)[0]

    @classmethod
    def get_absolute_testcase_id(cls, testcase_id, scope):
        """
        Transform a possibly relative test case id to an absolute one using the scope in which it is used.

        >>> scope = 'test_module.py.TestClass.test'
        >>> get_absolute_nodeid('test2', scope)
        'test_module.py.TestClass.test2'
        >>> get_absolute_nodeid('TestClass2.test2', scope)
        'test_module.py.TestClass2.test2'
        >>> get_absolute_nodeid('test_module2.py.TestClass2.test2', scope)
        'test_module2.py.TestClass2.test2'
        """

        tid = testcase_id
        parts = tid.split(cls.SEP)
        id_len = len(parts)
        # Completely relative (test_name), so add the full current scope
        if id_len == 1:
            base_tid = scope.rsplit(cls.SEP, 1)[0]
            tid = cls.SEP.join([base_tid, tid])
        # Contains some scope already (TestClass::test_name), so only add the current file scope
        elif id_len == 2 and '{}{}'.format(cls.SEP, parts[-1]) != cls.PY:
            base_tid = cls.SEP.join(scope.split(cls.SEP)[0:2])
            tid = cls.SEP.join([base_tid, tid])
        return tid

    def get_names(self):
        """
        Get all names for a test.

        This will use the following methods to determine the name of the test:
            - If given, the custom name(s) passed to the keyword argument name on test case settings
            - The full id of the test case
            - All 'scope' parts of the test case id. For example, for a test test_file.py.TestClass.test, this would be
            test_file.py and test_file.py.TestClass
        """

        names = set()

        # test case id
        names.add(self.new_id)

        # 用例所属的域名
        scopes = [self.module_scope, self.class_scope]
        for scope in scopes:
            names.add(scope)

        # Custom name
        for name in self.dnames:
            names.add(name)

        return names


class TestResultFinder(AttributeManager):

    UNRUN_CODE = ConstAttributeMarker(0, "未运行", alias="未运行")
    PASS_CODE = ConstAttributeMarker(1, "通过", alias="通过")
    FAIL_CODE = ConstAttributeMarker(2, "失败", alias="失败")
    ERROR_CODE = ConstAttributeMarker(3, "异常", alias="异常")

    @classmethod
    def find_test_result(cls, test, results):

        # find pass test case
        for ptest in results.successes:
            if test.id() == ptest.id():
                return (cls.PASS_CODE, cls.get_result_name(cls.PASS_CODE), '')
        for ptest, message in results.expectedFailures:
            if test.id() == ptest.id():
                return (cls.PASS_CODE, cls.get_result_name(cls.PASS_CODE), message)

        # find fail test case
        for ftest, message in results.failures:
            if test.id() == ftest.id():
                return (cls.FAIL_CODE, cls.get_result_name(cls.FAIL_CODE), message)
        for ftest in results.unexpectedSuccesses:
            if test.id() == ftest.id():
                return (cls.FAIL_CODE, cls.get_result_name(cls.FAIL_CODE), '')

        for etest, message in results.errors:
            if test.id() == etest.id():
                return (cls.ERROR_CODE, cls.get_result_name(cls.ERROR_CODE), message)
        return (cls.UNRUN_CODE, cls.get_result_name(cls.UNRUN_CODE), '')

    @classmethod
    def is_pass(cls, result_code):

        return result_code == cls.PASS_CODE

    @classmethod
    def get_result_name(cls, result_code):

        markers = cls.const_attrs.values()
        for marker in markers:
            code = marker.value
            if result_code == code:
                return marker.expend_kwargs["alias"]

    @classmethod
    def result_codes(cls):
        markers = cls.const_attrs.values()
        codes = [marker.value for marker in markers]
        return codes

    @classmethod
    def is_test_pass(cls, test, results):

        cval, cname, m = cls.find_test_result(test, results)
        return cls.PASS_CODE == cval


class TestDepends(object):
    """ Information about the resolved dependencies of a single test """
    def __init__(self, testcase_wrapper, manager):

        self.testid = testcase_wrapper.id()
        self.depends = set()
        self.unresolved = set()

        for depend in testcase_wrapper.depends:
            if depend not in manager.name_to_testids:
                absolute_depend = testcase_wrapper.get_absolute_testcase_id(depend, testcase_wrapper.new_id)
                if absolute_depend in manager.name_to_testids:
                    depend = absolute_depend

            if depend in manager.name_to_testids:
                for testid in manager.name_to_testids[depend]:
                    self.depends.add(testid)
            else:
                self.unresolved.add(depend)


class DependsManager(object):
    def __init__(self):

        self._tests = None
        self._results = None
        self._name_to_testids = None
        self._testid_to_test = None
        self._weights = {}
        self._links = {}
        self._cyclic_links = {}

    @property
    def tests(self):

        if self._tests is None:
            raise AttributeError('The tests attribute has not been set yet')
        return self._tests

    @tests.setter
    def tests(self, testcases):

        if self._tests is not None:
            raise AttributeError('The tests attribute has already been set')
        self._tests = testcases

        self._depends = {}
        self._testid_to_test = {}
        self._name_to_testids = collections.defaultdict(list)

        tc_wrappers = [TestCaseWrapper(tc) for tc in testcases]

        for tc in tc_wrappers:
            self._testid_to_test[tc.id()] = tc.testcase
            for name in tc.get_names():
                self._name_to_testids[name].append(tc.id())

        # Don't allow using unknown keys on the name_to_testids mapping
        self._name_to_testids.default_factory = None

        for tc in tc_wrappers:
            self._depends[tc.id()] = TestDepends(tc, self)

        for tc in tc_wrappers:
            weight, links, cyclic_links = self.calc_weight_and_link(tc.id())
            self._weights[tc.id()] = weight
            self._links[tc.id()] = links
            self._cyclic_links[tc.id()] = cyclic_links

    def calc_weight_and_link(self, testcase_id):
        """ 计算用例的依赖权重和他所依赖的用例（包括直接和间接依赖），同时检查是否存在循环依赖

        Args:
            testcase_id: 测试用例id

        Returns: 返回3个元素的元祖，第一个是权重，第二个是所依赖的用例id列表，第三个是循环依赖的用例id列表

        Useage:
            t1 -> t2 -> t3    t1 -> t2 ->t4 ->t5

            self.calc_weight_and_link(t1.id)

            >>> (4, [[t2.id, t3.id],[t2.id, t4.id, t5.id]], [])

            t1 -> t2 -> t3    t1 -> t2 ->t4 ->t1

            self.calc_weight_and_link(t1.id)

            >>> (4, [[t2.id, t3.id],[t2.id, t4.id]], [[t1.id, t2.id, t4.id, t1.id]])
        """

        count = 0
        cyclic_links = []

        def _calc_weight_and_link(testid, link_recorder=[]):

            weight = 0
            links = []
            nonlocal count
            nonlocal cyclic_links
            depends_info = self.depends.get(testid, None)
            count = count + 1
            link_recorder.append(testid)

            if depends_info is not None:
                know_depends = depends_info.depends
                unknow_depends = depends_info.unresolved
                kcounts = len(know_depends)
                ucounts = len(unknow_depends)
                weight = weight + kcounts + ucounts
                if kcounts > 0:
                    for test_id in know_depends:
                        if test_id in link_recorder:  # 说明当前依赖关系链路已造成循环依赖
                            cyclic_link = [c for c in link_recorder]
                            cyclic_link.append(test_id)
                            cyclic_links.append(cyclic_link)
                            link_recorder.pop()  # 依赖链路死循环，获取父级链路，以便记录其他兄弟节点的链路

                            # continue # 原码 为了获取所有的循环链，但多参数化传入多条数据进行数据驱动测试，设置循环依赖会导致代码死循环，无法获取循环链 2021-09-29 注释 by 思文伟
                            break  # 由continue 改为break  2021-09-29 注释 by 思文伟

                        sub_weight, sub_links = _calc_weight_and_link(test_id, link_recorder)
                        weight = weight + sub_weight
                        link = []
                        if sub_links:
                            for slink in sub_links:
                                onelink = [test_id]
                                onelink.extend(slink)
                                link.append(onelink)
                        else:
                            link.append([test_id])

                            # 依赖链路已经结束，获取父级链路，以便记录其他兄弟节点的链路
                            if link_recorder[-1] == test_id:
                                link_recorder.pop()
                        if link:
                            links.extend(link)
            return (weight, links)

        r = _calc_weight_and_link(testcase_id)
        return (r[0], r[1], cyclic_links)

    @property
    def name_to_testids(self):
        """ A mapping from names to matching test case id(s). """
        assert self.tests is not None
        return self._name_to_testids

    @property
    def testid_to_test(self):
        """ A mapping from node ids to test cases. """

        assert self.tests is not None
        return self._testid_to_test

    def sorted_tests(self, suiteclass=None):
        """ Get a sorted list of tests where all tests are sorted after their dependencies. """

        ztests = [test for test in self.tests if self.weights[test.id()] == 0]
        dtests = [test for test in self.tests if self.weights[test.id()] > 0]
        dtests.sort(key=lambda test: self.weights[test.id()])

        tests = []
        tests.extend(ztests)
        tests.extend(dtests)
        if suiteclass is not None:
            tests = suiteclass(tests)
        return tests

    @property
    def results(self):

        if self._results is None:
            raise AttributeError('The results attribute has not been set yet')
        return self._results

    @results.setter
    def results(self, value):

        self._results = value

    def get_depend_tests(self, test):
        """ 获取测试用例所依赖的其它用例"""

        dtests = []
        test_depend = self.depends[test.id()]
        for dtestid in test_depend.depends:
            dtests.append(self.testid_to_test[dtestid])
        return dtests

    def dependent_test_is_pass(self, test, results=None):
        """ 检查测试用例所依赖的其他测试用例是否测试通过，只要有一个不通过则返回False，否则为True，同时返回失败的详情信息列表"""

        is_pass = True
        sep = '------->'
        fail_details = []
        results = results or self.results
        finder = TestResultFinder()
        for dtest in self.get_depend_tests(test):
            codevalue, codename, message = finder.find_test_result(dtest, results)
            if not finder.is_pass(codevalue):
                is_pass = False
                msg = '{} {} {}({})'.format(dtest.id(), sep, codename, codevalue)
                fail_details.append(msg)
        return (is_pass, fail_details)

    def is_test_pass(self, test, results=None):
        """ 检查 测试用例所依赖的其它测试用例是否通过"""

        results = results or self.results
        finder = TestResultFinder()
        return finder.is_test_pass(test, results)

    @property
    def weights(self):
        """ the weight of  the tests"""

        assert self.tests is not None
        return self._weights

    @property
    def links(self):
        """ the depend chains of the tests"""

        assert self.tests is not None
        return self._links

    @property
    def depends(self):
        """ The dependencies of the tests. """

        assert self.tests is not None
        return self._depends

    def get_missing(self, test):

        return self.depends[test.id()].unresolved

    @property
    def cyclic_links(self):

        assert self.tests is not None
        return self._cyclic_links

    def get_cyclic_links(self, test):

        return self.cyclic_links[test.id()]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/03/30 11:21:57
'''
import sys
import time
import inspect
import unittest
from .utils import strclass
from .test_wrapper import Test


class AbstractTestCase(unittest.TestCase):
    """为测试类添加自动收集和自行测试用例的方法"""
    def __init__(self, methodName='runTest', serial_number=1):
        super(AbstractTestCase, self).__init__(methodName)
        self._serial_number = serial_number
        self.__testcase_runtime_datas = dict(args=[], kwargs={})

    @classmethod
    def sleep(cls, seconds):

        time.sleep(seconds)
        return cls

    def strclass(self):
        return strclass(self.__class__)

    @property
    def serial_number(self):
        return self._serial_number

    @property
    def test_method_obj(self):
        return getattr(self, self._testMethodName)

    @property
    def real_test_method_name(self):
        return self._testMethodName

    @property
    def test_method_settings(self):

        return Test.get_test_marker(self.test_method_obj)

    @property
    def _test_method_name(self):

        return "{}_{}".format(self._testMethodName, self._serial_number)

    def set_testcase_runtime_datas(self, args=[], kwargs={}):

        self.__testcase_runtime_datas["args"] = args
        self.__testcase_runtime_datas["kwargs"] = kwargs

    def get_testcase_runtime_datas(self):
        return self.__testcase_runtime_datas

    def shortDescription(self):
        name = Test.get_test_marker(self.test_method_obj, key=Test.DESCRIPTION, default_value=None)
        return name or None

    def id(self):
        return "{}.{}".format(strclass(self.__class__), self._test_method_name)

    def __eq__(self, other):
        if type(self) is not type(other):
            return NotImplemented

        return self._test_method_name == other._test_method_name

    def __hash__(self):
        return hash((type(self), self._test_method_name))

    def __str__(self):
        return "%s (%s)" % (self._test_method_name, strclass(self.__class__))

    def __repr__(self):
        return "<%s testMethod=%s>" % \
               (strclass(self.__class__), self._test_method_name)

    def run(self, result=None):

        if result is not None:
            dm = getattr(result, 'depend_manager', None)
            # 如果有设置依赖管理器，则运行依赖管理器
            if dm is not None:

                addFailure = getattr(result, 'addFailure', None)

                # 检查是否有无法解析的依赖
                missing = dm.get_missing(self)
                miss_msg = '{} depends on {}, which was not found'.format(self.id(), ", ".join(missing))
                if missing:
                    try:
                        raise self.failureException(miss_msg)
                    except self.failureException:
                        exc_info = sys.exc_info()
                        if addFailure:
                            addFailure(self, exc_info)
                    return result

                # 检查是否存在循环依赖
                clinks = dm.get_cyclic_links(self)
                printables = []
                for clink in clinks:
                    printable = '------>'.join(clink)
                    printables.append(printable)
                if clinks:
                    try:
                        raise self.failureException('{} circular dependency: {}'.format(self.id(), ", ".join(printables)))
                    except self.failureException:
                        exc_info = sys.exc_info()
                        if addFailure:
                            addFailure(self, exc_info)
                    return result

                # alway_run 为True，则不管该用例所依赖的其他用例是否成功都会执行该用例
                alway_run = self.test_method_settings.get(Test.ALWAY_RUN, False)
                if alway_run:
                    return super().run(result=result)

                # 检查用例所依赖的其他测试用例是否测试通过，如果不通过则不执行该用例并标记结果为失败
                tresult, msglist = dm.dependent_test_is_pass(self)
                if not tresult:
                    try:
                        raise self.failureException('{} depends on: {}'.format(self.id(), '\n'.join(msglist)))
                    except self.failureException:
                        exc_info = sys.exc_info()
                        if addFailure:
                            addFailure(self, exc_info)
                    return result
        return super().run(result=result)

    @classmethod
    def collect_testcases(cls):

        members = [obj_val for obj_key, obj_val in cls.__dict__.items() if inspect.ismethod(obj_val) or inspect.isfunction(obj_val)]
        test_func_list = [member for member in members if Test.func_has_test_marker(member)]
        run_test_func_list = [tf for tf in test_func_list if Test.get_test_marker(tf, key=Test.ENABLED, default_value=False)]
        run_test_func_list.sort(key=lambda tf: Test.get_test_marker(tf, key=Test.PRIORITY, default_value=1))
        testcases = []
        for test_func in run_test_func_list:
            test_func.collect_test_datasets(cls, test_func)
            datasets = test_func.test_settings[Test.TEST_DATASETS]
            if (len(datasets) > 0):
                for i, v in enumerate(datasets):
                    testcases.append(cls(test_func.__name__, (i + 1)))
            else:
                testcases.append(cls(test_func.__name__))
        return testcases

    @classmethod
    def build_self_suite(cls):

        suite = unittest.TestSuite()
        suite.addTests(cls.collect_testcases())
        return suite

    @classmethod
    def run_test(cls, **kwargs):

        from stest.main import main
        task = main(**kwargs)
        return task.result



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import setuptools

with open('README.md', 'r', encoding='UTF-8') as f:
    long_description = f.read()

setuptools.setup(
    name="stest",
    version="1.0.1",
    author="思文伟",
    author_email="hotswwkyo@qq.com",
    description="基于unittest开发的测试框架，更友好、更灵活的编写、管理与运行测试，生成更加美观的独立单文件HTML报告。内置参数化测试数据存取方案，省去设计的烦恼，节省更多的时间，从而更快的投入到编写用例阶段",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/hotswwkyo/stest",
    packages=setuptools.find_packages(),
    install_requires=["xlrd==1.2.0"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: Chinese (Simplified)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3',
    package_data={
        "stest": ["samples/*.xlsx", "report/resources/*.*"],
    },
)



# -*- coding: utf-8 -*-
'''
Created on 2019年12月5日

@author: siwenwei
'''

from .tags import Tags


class HtmlElement(object):
    def __init__(self, tag, text="", has_end_tag=True, **attributes):

        self._set_tag(tag)
        self._has_end_tag = has_end_tag
        self._attributes = attributes
        self._text = text
        self._child_elements = []
        self._following_sibling_elements = []
        self._preceding_sibling_elements = []

    @property
    def tag(self):
        return self._tag

    def _set_tag(self, tag):

        if tag and isinstance(tag, str):
            self._tag = tag
        else:
            raise TypeError("tag name must be string type")

    @property
    def child_elements(self):
        return self._child_elements

    @property
    def is_empty_element(self):
        """判断是否空元素（即没有结束标签的元素）"""

        return False if self._has_end_tag else True

    def rename_tag(self, tag):

        self._set_tag(tag)
        return self

    @property
    def text(self):

        return self._text

    @text.setter
    def text(self, value):

        self._text = value

    @property
    def attributes(self):

        return self._attributes

    def set_attr(self, name, value):

        self._attributes[name] = value
        return self

    def get_attr(self, name):

        return self._attributes[name]

    def set_attributes(self, **attributes):

        self._attributes.update(attributes)
        return self

    def remove_attributes(self, **attributes):

        for k in attributes.keys():
            if k in self._attributes.keys():
                self._attributes.pop(k)
        return self

    def add_css_class(self, *css_classes):

        vaild_css_classes = []
        for css_cls in css_classes:
            if isinstance(css_cls, str):
                vaild_css_classes.append(css_cls)
        if not vaild_css_classes:
            return self
        attr_name = "class"
        exists_css_classes = self._attributes.get(attr_name, "").strip()
        if exists_css_classes:
            css_class_list = exists_css_classes.split()
        else:
            css_class_list = []
        for css_class in vaild_css_classes:
            if " " not in css_class:
                css_class_list.append(css_class)
        value = " ".join(css_class_list)
        self.set_attr(attr_name, value)
        return self

    def clear_all_attributes(self):

        self._attributes.clear()
        return self

    def _validate_element(self, element):

        if not isinstance(element, HtmlElement):
            raise TypeError("elements is not html element")

    def remove_child_by_tag(self, tag):

        for el in self.get_child_elements_by_tag(tag):
            self._child_elements.remove(el)
        if self.get_child_elements_by_tag(tag):
            self.remove_child_by_tag(tag)
        return self

    def insert_child(self, child_element, index=0):

        self._child_elements.insert(index, child_element)

    def append_child(self, *child_elements):

        for child_element in child_elements:
            self._validate_element(child_element)
            if self.is_empty_element:
                self._following_sibling_elements.append(child_element)
            else:
                self._child_elements.append(child_element)
        return self

    def before(self, *preceding_sibling_elements):

        for element in preceding_sibling_elements:
            self._validate_element(element)
            self._preceding_sibling_elements.append(element)

    def after(self, *following_sibling_elements):

        for element in following_sibling_elements:
            self._validate_element(element)
            self._following_sibling_elements.append(element)

    def _element_to_html_text_list(self, element_list):

        return [el.to_html() for el in element_list]

    def _html_of_child_elements(self):

        return self._element_to_html_text_list(self._child_elements)

    def _attributes_to_text(self):

        if self._attributes:
            sep = " "
        else:
            sep = ""
        return sep.join(['%s="%s"' % (k, v) for k, v in self._attributes.items()])

    def get_child_elements_by_tag(self, tag):

        return [el for el in self._child_elements if el.tag == tag]

    def _build_normal_element(self):

        attrs = self._attributes_to_text()
        text = str(self._text) + "".join(self._html_of_child_elements())
        el_html = "<{tag} {attrs}>{text}</{tag}>".format(tag=self._tag, attrs=attrs, text=text)
        return el_html

    def _build_empty_element(self):
        """创建没有结束标签的元素"""

        attrs = self._attributes_to_text()
        el_html = "<{tag} {attrs} />".format(tag=self._tag, attrs=attrs)
        return el_html

    def to_html(self):

        preceding_siblings = self._element_to_html_text_list(self._preceding_sibling_elements)
        following_siblings = self._element_to_html_text_list(self._following_sibling_elements)

        if not self.is_empty_element:
            itself = self._build_normal_element()
        else:
            itself = self._build_empty_element()
        return "".join(preceding_siblings) + itself + "".join(following_siblings)


class HtmlNormalElement(HtmlElement):
    def __init__(self, tag, text=""):

        super().__init__(tag, text, has_end_tag=True)


class HtmlEmptyElement(HtmlElement):
    def __init__(self, tag):

        super().__init__(tag, "", has_end_tag=False)


class HTML(HtmlNormalElement):

    DOCTYPE = """<!DOCTYPE html>"""

    def __init__(self, text=""):
        super().__init__(Tags.html, text)
        self._allow_element_exclude_head_and_body = False
        self.append_child(Head())
        self.append_child(Body())

    @property
    def allow_add(self):
        return self._allow_element_exclude_head_and_body

    @allow_add.setter
    def allow_add(self, value):

        self._allow_element_exclude_head_and_body = value

    def _has_head(self):

        return True if self.get_child_elements_by_tag(Tags.head) else False

    def _has_body(self):
        return True if self.get_child_elements_by_tag(Tags.body) else False

    def _get_other_child_elements(self, child_elements):

        return [el for el in child_elements if el.tag != Tags.body and el.tag != Tags.head]

    def append_child(self, *child_elements):

        if not self._has_head():
            heads = [el for el in child_elements if el.tag == Tags.head]
            if heads:
                HtmlElement.append_child(self, heads[0])

        if not self._has_body():
            bodys = [el for el in child_elements if el.tag == Tags.body]
            if bodys:
                HtmlElement.append_child(self, bodys[0])

        other_child_elements = self._get_other_child_elements(child_elements)

        if not other_child_elements:
            return self
        if not self._allow_element_exclude_head_and_body:
            message = """"HTML already has head and body elements.
            It is not recommended to add any direct child elements.
            if you need to add, set self.allow_add = True"""
            raise Warning(message)
        return HtmlElement.append_child(self, *other_child_elements)

    @property
    def head(self):
        elements = self.get_child_elements_by_tag(Tags.head)
        return elements[0] if elements else None

    @property
    def body(self):
        elements = self.get_child_elements_by_tag(Tags.body)
        return elements[0] if elements else None


class Head(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.head, text)
        self.append_child(Title())

    @property
    def title(self):
        elements = self.get_child_elements_by_tag(Tags.title)
        return elements[0] if elements else None


class Title(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.title, text)


class Meta(HtmlEmptyElement):
    def __init__(self):
        super().__init__(Tags.meta)


class Style(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.style, text)


class Script(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.script, text)


class Body(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.body, text)


class Table(HtmlNormalElement):
    def __init__(self, text=""):

        super().__init__(Tags.table, text)

    def _is_only_one_thead_and_tbody(self, child_elements):

        thead_count = 0
        tbody_count = 0
        for el in child_elements:

            if thead_count > 1 or tbody_count > 1:
                raise ValueError("Table can only have one thead and one tbody")

            if el.tag == Tags.thead:
                thead_count = thead_count + 1
            elif el.tag == Tags.tbody:
                tbody_count = tbody_count + 1

    def append_child(self, *child_elements):

        self._is_only_one_thead_and_tbody(child_elements)
        return HtmlNormalElement.append_child(self, *child_elements)

    @property
    def thead(self):

        elements = self.get_child_elements_by_tag(Tags.thead)
        return elements[0] if elements else None

    @property
    def tbody(self):

        elements = self.get_child_elements_by_tag(Tags.tbody)
        return elements[0] if elements else None

    @property
    def header_rows(self):

        rows = self.get_child_elements_by_tag(Tags.tr)
        hrows = []
        for row in rows:
            if row.get_child_elements_by_tag(Tags.th):
                hrows.append(row)
        if self.thead:
            hrows.extend(self.thead.rows)
        return hrows

    @property
    def body_rows(self):

        rows = self.get_child_elements_by_tag(Tags.tr)
        brows = []
        for row in rows:
            if row.get_child_elements_by_tag(Tags.td):
                brows.append(row)
        if self.tbody:
            brows.extend(self.tbody.rows)
        return brows

    @property
    def rows(self):

        return self.header_rows + self.body_rows

    @property
    def nrows(self):

        return len(self.header_rows) + len(self.body_rows)

    @property
    def ncols(self):

        ncols_of_rows = [row.ncols for row in self.rows]
        return max(ncols_of_rows) if ncols_of_rows else 0

    def _build_header(self):

        pass

    def _build_body(self):

        pass

    def to_html(self):

        self._build_header()
        self._build_body()
        return HtmlNormalElement.to_html(self)


class Thead(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.thead, text)

    @property
    def rows(self):
        return self.get_child_elements_by_tag(Tags.tr)

    @property
    def nrows(self):
        return len(self.rows)

    @property
    def ncols(self):
        ncols_of_row = [row.ncols for row in self.rows]
        return max(ncols_of_row) if ncols_of_row else 0


class Tbody(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.tbody, text)

    @property
    def rows(self):
        return self.get_child_elements_by_tag(Tags.tr)

    @property
    def nrows(self):
        return len(self.rows)

    @property
    def ncols(self):
        ncols_of_row = [row.ncols for row in self.rows]
        return max(ncols_of_row) if ncols_of_row else 0


class TH(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.th, text)


class TR(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.tr, text)

    @property
    def cols(self):

        return self.get_child_elements_by_tag(Tags.td) or self.get_child_elements_by_tag(Tags.th)

    @property
    def ncols(self):

        return len(self.cols)


class TD(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.td, text)


class Div(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.div, text)


class Span(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.span, text)


class Fieldset(HtmlNormalElement):
    def __init__(self, text=""):
        """

        @param text: 组标题
        """
        super().__init__(Tags.fieldset, "")
        self.append_child(Legend(text))

    @property
    def legend(self):

        return self.get_child_elements_by_tag(Tags.legend)[0]

    @property
    def title(self):
        return self.legend.text

    @title.setter
    def title(self, value):
        self.legend.text = value
        return self

    def append_child(self, *child_elements):

        for el in child_elements:
            if el.tag != Tags.legend:
                HtmlNormalElement.append_child(self, el)
            else:
                if not self.get_child_elements_by_tag(Tags.legend):
                    HtmlNormalElement.append_child(self, el)
        return self


class Legend(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.legend, text)


class Label(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.label, text)


class Pre(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.pre, text)


class P(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.p, text)


class Svg(HtmlNormalElement):
    class Path(HtmlEmptyElement):
        def __init__(self):
            super().__init__(Tags.path)
            self.__commands = []

        @property
        def commands(self):

            return self.__commands

        def M(self, x, y):
            """构建移动到x, y坐标点指令"""

            cmd = "M {x},{y}".format(x=x, y=y)
            self.commands.append(cmd)
            return self

        def L(self, x, y):
            """构建画一条直线到x, y坐标点指令"""

            cmd = "L {x},{y}".format(x=x, y=y)
            self.commands.append(cmd)
            return self

        def A(self, x, y, rx, ry, large_arc_flag, x_axis_rotation="0", sweep_flag="1"):
            """构建弧度指令

            Args:
                x: 弧线终点x坐标
                y: 弧线终点y坐标
                rx: x轴半径
                ry: y轴半径
                large_arc_flag: 角度大小 决定弧线是大于还是小于180度，0表示小角度弧，1表示大角度弧
                x_axis_rotation: 弧形与 x 轴旋转角度
                sweep_flag: 弧线的方向，0表示从起点到终点沿逆时针画弧，1表示从起点到终点沿顺时针画弧
            """

            cmd = "A {rx},{ry}, {x_axis_rotation} {large_arc_flag} {sweep_flag} {x},{y}".format(rx=rx, ry=ry, x_axis_rotation=x_axis_rotation, large_arc_flag=large_arc_flag, sweep_flag=sweep_flag, x=x, y=y)
            self.commands.append(cmd)
            return self

        def Z(self):

            cmd = "Z"
            self.commands.append(cmd)
            return self

        def set_d_attr(self):

            space = " "
            d_cmd = space.join(self.commands)
            self.set_attr("d", d_cmd)
            return d_cmd

    class Circle(HtmlEmptyElement):
        def __init__(self):
            super().__init__(Tags.circle)

    class Rect(HtmlEmptyElement):
        def __init__(self):
            super().__init__(Tags.rect)

    class Text(HtmlNormalElement):
        def __init__(self, text=""):
            super().__init__(Tags.text, text)

    def __init__(self, width="300", height="150"):

        super().__init__(Tags.svg)
        self.set_attributes(width=width, height=height)
        self.__svg_paths = {}
        self.__svg_rects = {}
        self.__svg_texts = {}

    @property
    def svg_paths(self):
        return self.__svg_paths

    @property
    def svg_rects(self):
        return self.__svg_rects

    @property
    def svg_texts(self):
        return self.__svg_texts

    def svg_path_is_exists(self, name):

        return name in self.svg_paths

    def get_svg_path(self, name):
        return self.svg_paths[name]

    def draw_svg_path(self, name):

        svg_path = self.Path()
        self.svg_paths[name] = svg_path
        self.append_child(svg_path)
        return svg_path

    def draw_svg_circle(self, name):
        svg_path = self.Circle()
        self.svg_paths[name] = svg_path
        self.append_child(svg_path)
        return svg_path

    def draw_svg_rect(self, name):

        svg_rect = self.Rect()
        self.svg_rects[name] = svg_rect
        self.append_child(svg_rect)
        return svg_rect

    def draw_svg_text(self, name, text=""):

        svg_text = self.Text(text)
        self.svg_texts[name] = svg_text
        self.append_child(svg_text)
        return svg_text


class DL(HtmlNormalElement):
    def __init__(self):
        super().__init__(Tags.dl)


class DT(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.dt, text=text)


class DD(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.dd, text=text)


class Input(HtmlEmptyElement):
    def __init__(self, itype="text", value=""):
        super().__init__(Tags.input)
        self.set_attr("type", itype)
        self.set_attr("value", value)


class HR(HtmlEmptyElement):
    def __init__(self):
        super().__init__(Tags.hr)


class UL(HtmlNormalElement):
    def __init__(self):
        super().__init__(Tags.ul)


class LI(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.li, text)


class Textarea(HtmlNormalElement):
    def __init__(self, text=""):
        super().__init__(Tags.textarea, text)


class Italic(HtmlNormalElement):
    def __init__(self, *css_classes):
        super().__init__(Tags.italic)
        self.add_css_class(*css_classes)



# -*- coding: utf-8 -*-
'''
Created on 2019年12月5日

@author: siwenwei
'''

from ..html import elements
from .piechart import PieChart


class SummaryTable(elements.Table):

    FIXED_WIDTH = "fixed-width"
    DEFAULT_CSS_CLASSES = ["seven-table", "summary"]
    PIE_CHART_FAIL_LABEL = "失败"
    PIE_CHART_FAIL_COLOR = "red"
    PIE_CHART_FAIL_CSS_CLASSES = {"pie_chart_css_class": "fail", "legend_css_class": "fail-legend", "legend_text_css_class": "fail-legend-text"}
    PIE_CHART_BLOCK_LABEL = "阻塞"
    PIE_CHART_BLOCK_COLOR = "sandybrown"
    PIE_CHART_BLOCK_CSS_CLASSES = {"pie_chart_css_class": "block", "legend_css_class": "block-legend", "legend_text_css_class": "block-legend-text"}

    PIE_CHART_ERROR_LABEL = "异常"
    PIE_CHART_ERROR_COLOR = "blueviolet"
    PIE_CHART_ERROR_CSS_CLASSES = {"pie_chart_css_class": "error", "legend_css_class": "error-legend", "legend_text_css_class": "error-legend-text"}

    PIE_CHART_SUCCESS_LABEL = "通过"
    PIE_CHART_SUCCESS_COLOR = "green"
    PIE_CHART_SUCCESS_CSS_CLASSES = {"pie_chart_css_class": "success", "legend_css_class": "success-legend", "legend_text_css_class": "success-legend-text"}

    PIE_CHART_SKIP_LABEL = "skip"
    PIE_CHART_SKIP_COLOR = "#c0c0c0"
    PIE_CHART_SKIP_CSS_CLASSES = {"pie_chart_css_class": "skip", "legend_css_class": "skip-legend", "legend_text_css_class": "skip-legend-text"}

    PIE_CHART_XFAIL_LABEL = "预期失败"
    PIE_CHART_XFAIL_COLOR = "#9acd32"
    PIE_CHART_XFAIL_CSS_CLASSES = {"pie_chart_css_class": "xfail", "legend_css_class": "xfail-legend", "legend_text_css_class": "xfail-legend-text"}

    PIE_CHART_XPASS_LABEL = "unexpected passes"
    PIE_CHART_XPASS_COLOR = "#a52a2a"
    PIE_CHART_XPASS_CSS_CLASSES = {"pie_chart_css_class": "xpass", "legend_css_class": "xpass-legend", "legend_text_css_class": "xpass-legend-text"}

    SVG_BOX_CSS_CLASSES = ["svg-box"]

    def __init__(self, title="", task_number="", start_time="", finish_time="", executor="", project_name="", task_description="", pie_chart_info=()):
        super().__init__()
        self.add_css_class(*self.DEFAULT_CSS_CLASSES)
        self.append_child(elements.Thead(), elements.Tbody())
        self.title = title
        self.task_number = task_number
        self.start_time = start_time
        self.finish_time = finish_time
        self.executor = executor
        self.project_name = project_name
        self.task_description = task_description
        self.pie_chart_info = pie_chart_info

    def _build_header(self):
        row = elements.TR()
        colspan = 3 if self.pie_chart_info else 2
        task_title_cell = elements.TD(self.title).add_css_class(self.FIXED_WIDTH).set_attr("colspan", colspan)
        row.append_child(task_title_cell)
        self.thead.append_child(row)

    def _build_body(self):
        self._set_task_number(self.task_number)
        self._set_project_name(self.project_name)
        self._set_executor(self.executor)
        self._set_start_time(self.start_time)
        self._set_finish_time(self.finish_time)
        self._set_task_description(self.task_description)

    def _set_task_number(self, task_number):

        title = "测试任务"
        row, title_col, value_col = self._build_row(title, task_number)
        if self.pie_chart_info and max(self.pie_chart_info) > 0:
            width = 500
            height = 260
            cx = 100
            cy = 100
            r = 100
            pass_count, fail_count, block_count, error_count, skip_count, xfail_count, xpass_count = self.pie_chart_info
            parts = [
                PieChart.build_part(pass_count, self.PIE_CHART_SUCCESS_LABEL, self.PIE_CHART_SUCCESS_COLOR, **self.PIE_CHART_SUCCESS_CSS_CLASSES),
                PieChart.build_part(fail_count, self.PIE_CHART_FAIL_LABEL, self.PIE_CHART_FAIL_COLOR, **self.PIE_CHART_FAIL_CSS_CLASSES),
                PieChart.build_part(block_count, self.PIE_CHART_BLOCK_LABEL, self.PIE_CHART_BLOCK_COLOR, **self.PIE_CHART_BLOCK_CSS_CLASSES),
                PieChart.build_part(error_count, self.PIE_CHART_ERROR_LABEL, self.PIE_CHART_ERROR_COLOR, **self.PIE_CHART_ERROR_CSS_CLASSES),
                PieChart.build_part(skip_count, self.PIE_CHART_SKIP_LABEL, self.PIE_CHART_SKIP_COLOR, **self.PIE_CHART_SKIP_CSS_CLASSES),
                PieChart.build_part(xfail_count, self.PIE_CHART_XFAIL_LABEL, self.PIE_CHART_XFAIL_COLOR, **self.PIE_CHART_XFAIL_CSS_CLASSES),
                PieChart.build_part(xpass_count, self.PIE_CHART_XPASS_LABEL, self.PIE_CHART_XPASS_COLOR, **self.PIE_CHART_XPASS_CSS_CLASSES),
            ]
            pie = PieChart(parts, width, height, cx, cy, r)
            pie_chart_column = elements.TD().set_attr("rowspan", 6)
            pie_chart_column.add_css_class(*self.SVG_BOX_CSS_CLASSES)
            pie_chart_column.append_child(pie)
            row.append_child(pie_chart_column)

        self.tbody.append_child(row)

    def _build_row(self, title, value):
        row = elements.TR()
        task_title_cell = elements.TD(title).add_css_class(self.FIXED_WIDTH)
        task_value_cell = elements.TD(value)
        row.append_child(task_title_cell)
        row.append_child(task_value_cell)
        return (row, task_title_cell, task_value_cell)

    def _set_task_description(self, task_description):

        title = "描述"
        row, title_col, value_col = self._build_row(title, task_description)
        self.tbody.append_child(row)

    def _set_project_name(self, project_name):

        title = "项目名称"
        row, title_col, value_col = self._build_row(title, project_name)
        self.tbody.append_child(row)

    def _set_executor(self, executor):

        title = "任务执行人"
        row, title_col, value_col = self._build_row(title, executor)
        self.tbody.append_child(row)

    def _set_start_time(self, start_time):

        title = "开始时间"
        row, title_col, value_col = self._build_row(title, start_time)
        self.tbody.append_child(row)

    def _set_finish_time(self, finish_time):

        title = "完成时间"
        row, title_col, value_col = self._build_row(title, finish_time)
        self.tbody.append_child(row)


class ReportTable(elements.Table):

    DEFAULT_CSS_CLASSES = ["seven-table"]
    FIXED_HEADER_TITLES = ["测试点 / 测试用例", "总计", "通过", "失败", "阻塞", "异常", "skip", "预期失败", "unexpected passes"]
    TESTPOINT_ID_PREFIX = "testpoint_"
    TESTCASE_ID_PREFIX = "testcase_"
    ID_SEP = "."
    TESTPOINT_ROW_CSS_CLASS = "testpoint"
    TESTPOINT_NAME_CSS_CLASS = "testpoint-name"
    TESTPOINT_COUNT_CSS_CLASS = "testpoint-count"
    TESTPOINT_PASS_CSS_CLASS = "testpoint-pass"
    TESTPOINT_FAIL_CSS_CLASS = "testpoint-fail"
    TESTPOINT_BLOCK_CSS_CLASS = "testpoint-block"
    TESTPOINT_ERROR_CSS_CLASS = "testpoint-error"

    TESTPOINT_SKIP_CSS_CLASS = "testpoint-skip"
    TESTPOINT_XFAIL_CSS_CLASS = "testpoint-xfail"
    TESTPOINT_XPASS_CSS_CLASS = "testpoint-xpass"

    TESTCASE_ROW_CSS_CLASS = ["testcase", "testcase-show"]
    TESTCASE_NAME_COL_CSS_CLASS = "testcase-name"
    TESTCASE_RESULT_COL_CSS_CLASS = "testcase-result"
    TESTSTEPS_ZONE_ROW_ID = "teststeps"
    TESTSTEPS_ZONE_ROW_CSS_CLASS = "teststeps"
    TESTSTEP_ID_PREFIX = "teststep_"
    TESTSTEP_TABLE_CONTAINER_CSS_CLASS = "seven-table-container"

    def __init__(self, testpoints=[]):
        super().__init__()
        self._header_titles = []
        self._testpoints = testpoints
        self._header_titles.extend(self.FIXED_HEADER_TITLES)
        self.append_child(elements.Thead(), elements.Tbody())
        self.add_css_class(*self.DEFAULT_CSS_CLASSES)

    def set_testpoints(self, *testpoints):

        self._testpoints = testpoints

    @property
    def header_titles(self):
        return self._header_titles

    def add_header_titles(self, title, index=None):

        if isinstance(index, int) and index >= 0:
            self._header_titles.insert(index, title)
        else:
            self._header_titles.append(title)
        return self

    def _build_header(self):

        tr = elements.TR()
        for col_title in self.header_titles:
            cell = elements.TH(col_title)
            tr.append_child(cell)
        self.thead.append_child(tr)
        return self

    def __build_arg_area(self, arguments_dict, label=""):

        box = elements.Div()
        showview = elements.Div()
        # box.append_child(elements.P(label), elements.HR(), showview)
        box.append_child(elements.P(label).set_attr("style", "margin-top: 21px;"), showview)
        for k, v in arguments_dict.items():
            fs = elements.Fieldset(k)
            styles = ["border-top-style: solid; border-top-width: 1px; border-left: none; border-right: none; border-bottom: none;"]
            styles.append("border-color: #e6e6e6;")
            styles.append("margin-left: 7px;")
            styles.append("margin-right: 7px;")
            fs.set_attr("style", "".join(styles))
            fs.append_child(elements.Pre(v).set_attr("style", "padding-left: 21px; padding-right: 21px;"))
            showview.append_child(fs)
        return box

    def _build_console_row(self, testcase_html_id, teststep_zone_html_id, testcase):
        """构建输出消息区域行"""

        message = [testcase["output_message"], testcase["error_message"]]
        testdatas = testcase['testdatas']
        args, kwargs = testdatas
        nargs = {}
        for k, v in args.items():
            name = " - ".join([str(k), v[0]])
            nargs[name] = v[1]

        row = elements.TR().set_attr("id", teststep_zone_html_id).add_css_class(self.TESTSTEPS_ZONE_ROW_CSS_CLASS)
        cell = elements.TD().set_attr("colspan", len(self.header_titles)).add_css_class(self.TESTSTEP_TABLE_CONTAINER_CSS_CLASS)
        div = elements.Div()
        outputmsg = '\n'.join(message) if message else ''
        div.append_child(elements.Pre(outputmsg))
        if nargs:
            cell.append_child(self.__build_arg_area(nargs, label="位置参数"))
        if kwargs:
            cell.append_child(self.__build_arg_area(kwargs, label="关键字参数"))
        cell.append_child(div)
        row.append_child(cell)
        return row

    def _build_testcases_rows(self, teststep_html_id, testcases):

        rows = []
        for index, tc in enumerate(testcases):

            tc_html_id = "%(tp_id)s%(sep)s%(prefix)s%(sn)s" % dict(tp_id=teststep_html_id, sep=self.ID_SEP, prefix=self.TESTCASE_ID_PREFIX, sn=(index + 1))
            teststep_zone_html_id = "%(tc_id)s%(sep)s%(sn)s" % dict(tc_id=tc_html_id, sep=self.ID_SEP, sn=self.TESTSTEPS_ZONE_ROW_ID)
            row = elements.TR().set_attr("id", tc_html_id).add_css_class(*self.TESTCASE_ROW_CSS_CLASS)
            row.set_attr("onclick", "toggleTestStepsRow('%s')" % teststep_zone_html_id)
            preceding_siblings = []
            name_cell = elements.TD(tc["name"])
            name_cell.add_css_class(self.TESTCASE_NAME_COL_CSS_CLASS)
            name_cell.set_attr("title", tc["method_name"])
            preceding_siblings.append(name_cell)
            row.append_child(*preceding_siblings)

            result_cell = elements.TD(tc["result"]["name"])
            result_cell.add_css_class(self.TESTCASE_RESULT_COL_CSS_CLASS)
            result_cell.set_attr("style", "cursor: pointer;")
            result_cell.set_attr("colspan", len(self.header_titles) - len(preceding_siblings))
            row.append_child(result_cell)
            row.after(self._build_console_row(tc_html_id, teststep_zone_html_id, tc))
            rows.append(row)
        return rows

    def build_testpoints_rows(self):

        rows = []
        for index, tp in enumerate(self._testpoints):
            tp_html_id = "%(prefix)s%(sn)s" % dict(prefix=self.TESTPOINT_ID_PREFIX, sn=(index + 1))
            row = elements.TR().set_attr("id", tp_html_id).add_css_class(self.TESTPOINT_ROW_CSS_CLASS)
            row.set_attr("onclick", "toggleTestCaseOfTestPoint('%s','%s','%s',%s)" % (tp_html_id, self.TESTCASE_ID_PREFIX, self.ID_SEP, len(tp["testcases"])))
            point_name = tp["name"]
            short_name = point_name.split(".")[-1]
            name_cell = elements.TD(short_name).add_css_class(self.TESTPOINT_NAME_CSS_CLASS)
            name_cell.set_attr("title", point_name)
            row.append_child(name_cell)
            row.append_child(elements.TD(tp["count"]).add_css_class(self.TESTPOINT_COUNT_CSS_CLASS))
            row.append_child(elements.TD(tp["pass_count"]).add_css_class(self.TESTPOINT_PASS_CSS_CLASS))
            row.append_child(elements.TD(tp["fail_count"]).add_css_class(self.TESTPOINT_FAIL_CSS_CLASS))
            row.append_child(elements.TD(tp["block_count"]).add_css_class(self.TESTPOINT_BLOCK_CSS_CLASS))
            row.append_child(elements.TD(tp["error_count"]).add_css_class(self.TESTPOINT_ERROR_CSS_CLASS))
            row.append_child(elements.TD(tp["skip_count"]).add_css_class(self.TESTPOINT_SKIP_CSS_CLASS))
            row.append_child(elements.TD(tp["xfail_count"]).add_css_class(self.TESTPOINT_XFAIL_CSS_CLASS))
            row.append_child(elements.TD(tp["xpass_count"]).add_css_class(self.TESTPOINT_XPASS_CSS_CLASS))
            row.after(*tuple(self._build_testcases_rows(tp_html_id, tp["testcases"])))
            rows.append(row)
        return rows

    def _build_body(self):
        self.tbody.append_child(*self.build_testpoints_rows())



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Author: 思文伟
@Date: 2021/09/17
'''
import os
import sys
import datetime
import unittest
from unittest.signals import installHandler
from .seven_runner import SevenTestRunner
from .seven_loader import SevenTestLoader
from .report_builder import ReportBuilder


class SevenTestProgram(unittest.TestProgram):
    def __init__(self,
                 module='__main__',
                 defaultTest=None,
                 argv=None,
                 testRunner=SevenTestRunner,
                 testLoader=SevenTestLoader(),
                 exit=True,
                 verbosity=1,
                 failfast=None,
                 catchbreak=None,
                 buffer=None,
                 warnings=None,
                 *,
                 tb_locals=False):

        super().__init__(module=module,
                         defaultTest=defaultTest,
                         argv=argv,
                         testRunner=testRunner,
                         testLoader=testLoader,
                         exit=exit,
                         verbosity=verbosity,
                         failfast=failfast,
                         catchbreak=catchbreak,
                         buffer=buffer,
                         warnings=warnings,
                         tb_locals=tb_locals)

    def _initArgParsers(self):

        super()._initArgParsers()
        self._main_parser.add_argument('-html', '--html-report', dest='html', help="html report file full path")
        self._main_parser.add_argument('-title', '--report-title', dest='title', default="", help="html report title")
        self._main_parser.add_argument('-project', '--project-name', dest='project', default="", help="test project name")
        self._main_parser.add_argument('-task', '--task-number', dest='task', help="test task number")
        self._main_parser.add_argument('-tester', '--tester-name', dest='tester', help="tester name")
        self._main_parser.add_argument('-taskinfo', '--task-info', dest='task_description', default="", help="task description")

        self._discovery_parser.add_argument('-html', '--html-report', dest='html', help="html report file full path")
        self._discovery_parser.add_argument('-title', '--report-title', dest='title', default="", help="html report title")
        self._discovery_parser.add_argument('-project', '--project-name', dest='project', default="", help="test project name")
        self._discovery_parser.add_argument('-task', '--task-number', dest='task', help="test task number")
        self._discovery_parser.add_argument('-tester', '--tester-name', dest='tester', help="tester name")
        self._discovery_parser.add_argument('-taskinfo', '--task-info', dest='task_description', default="", help="task description")

    def __build_html_report(self, result, start_time, finish_time):

        fname = self.html
        ext = '.html'
        notice = False
        if not fname:
            if self.module is None:
                # warning_message = '没有传入报告文件名，不会生成测试报告文件'
                # print(warning_message)
                return None
            else:
                filepath = os.path.abspath(self.module.__file__)
                fname = os.path.splitext(filepath)[0] + ext
                notice = True
        title_key = 'title'
        title = self.title
        summary_info = {}
        if not title:
            pathname, extname = os.path.splitext(fname)
            summary_info[title_key] = os.path.basename(pathname)
        summary_info['start_time'] = start_time.strftime("%Y-%m-%d %H:%M:%S")
        summary_info['finish_time'] = finish_time.strftime("%Y-%m-%d %H:%M:%S")
        summary_info['task_description'] = self.task_description
        summary_info['executor'] = self.tester if self.tester else ''
        summary_info['task_number'] = self.task if self.task else start_time.strftime("%Y%m%d%H%M%S%f")
        ReportBuilder(result).build_html_report(fname, **summary_info)
        if notice:
            print('html report file: {}'.format(fname))
        return fname

    def runTests(self):
        if self.catchbreak:
            installHandler()
        if self.testRunner is None:
            self.testRunner = SevenTestRunner
        if isinstance(self.testRunner, type):
            try:
                try:
                    testRunner = self.testRunner(verbosity=self.verbosity, failfast=self.failfast, buffer=self.buffer, warnings=self.warnings, tb_locals=self.tb_locals)
                except TypeError:
                    # didn't accept the tb_locals argument
                    testRunner = self.testRunner(verbosity=self.verbosity, failfast=self.failfast, buffer=self.buffer, warnings=self.warnings)
            except TypeError:
                # didn't accept the verbosity, buffer or failfast arguments
                testRunner = self.testRunner()
        else:
            # it is assumed to be a TestRunner instance
            testRunner = self.testRunner
        start_time = datetime.datetime.now()
        self.result = testRunner.run(self.test)
        finish_time = datetime.datetime.now()
        self.__build_html_report(self.result, start_time, finish_time)
        if self.exit:
            sys.exit(not self.result.wasSuccessful())


main = SevenTestProgram
